/**
 * Copyright  2002 Sun Microsystems, Inc. All rights reserved.
 */

package edu.cmu.fantango.ejb;

import java.util.*;
import java.io.*;
import javax.naming.*;
import java.sql.*;
import javax.sql.*;

import edu.cmu.fantango.util.*;

/**
 * A simple stateless bean for the Orders application. This bean implements one
 * business method as declared by the remote interface.
 */
public class OrdersEJB implements javax.ejb.SessionBean
{
    private  javax.ejb.SessionContext m_ctx = null;

    /**
     *	Stored customer username
     */
	private String s_Username;

    /**
     *	Stored order id
     */
	private String s_OrderID;

    /**
     *	Stored showing id
     */
	private String s_ShowingID;

    /**
     *	Stored date
     */
	private String s_Date;

    /**
     *	Stored time
     */
	private String s_Time;

    /**
     *	Stored showing date
     */
	private String s_ShowingDate;

    /**
     *	Stored showing time
     */
	private String s_ShowingTime;

    /**
     *	Stored room name
     */
	private String s_RoomName;

    /**
     *	Stored theater name
     */
	private String s_TheaterName;

    /**
     *	Stored movie title
     */
	private String s_MovieTitle;

    /**
     *	Stored tickets
     */
	private String s_Tickets;

    /**
     *	Stores result of query in String
     */
	private String s_Result;

    /**
     *	Stores delimiter
     */
	private String s_Delimiter;

    /**
     *	Stored if it is cancelled
     */
	private String s_Cancelled;


    /**
     *	Session Attributes the bean needs
     */
	private String[] s_SAttributes;

    /**
     *	Store error
     */
	private String s_Error;

    /**
     *	Page to load
     */
	private static String s_Goto;


    /**
     *	Page to load on success
     */
	private static final String s_GotoSuccess = "/Orders.jsp";

    /**
     *	Page to load on failure
     */
	private static final String s_GotoFailure = "/Register.jsp";

    /**
     *	Required by EJB spec.
     */
    public void Orders ( ) {
    }


	/**
	 *	Initializes Orders class to store and process
	 *	showing information.
	 *
	 *	@throws java.rmi.RemoteException
	 */
    public void Init ( ) throws java.rmi.RemoteException
    {
		System.out.println ( "::OrdersEJB:: (Init)" );
		s_SAttributes = new String[1];
		s_SAttributes[0] = "userloggedin";
		s_Username = "";
		s_OrderID = "";
		s_ShowingID = "";
		s_Date = "";
		s_Time = "";
		s_ShowingDate = "";
		s_ShowingTime = "";
		s_RoomName = "";
		s_TheaterName = "";
		s_MovieTitle = "";
		s_Tickets = "";
		s_Cancelled = "";
		s_Delimiter = "__";
		s_Goto = "";
    }


    /**
     *	Returns the delimiter used for form field separation
     *
     *	@returns Delimiter used for form field separation
     */
	private String getDelimiter ( )
	{
		return s_Delimiter;
	}


	/**
	 *	Initializes attributes from session.
	 *
	 *	@param	sattribs Values to assign to needed attributes
	 *	@throws java.rmi.RemoteException
	 */
    public void InitSAttribs ( String[] sattribs ) throws java.rmi.RemoteException
    {
		System.out.println ( "::OrdersEJB:: (InitSAttribs)" );
		s_Username = sattribs[0];
    }


	/**
	 *	Returns array of String attributes for the servlet to get
	 *	from session.
	 *
	 *	@throws java.rmi.RemoteException
	 *	@returns Array of strings of attribute names
	 */
	public String[] getSAttributes ( ) throws java.rmi.RemoteException
	{
		System.out.println ( "::OrdersEJB:: (getSAttributes)" );
		return s_SAttributes;
	}


	/**
	 *	Returns error state
	 *
	 *	@throws java.rmi.RemoteException
	 *	@returns Possible errors encountered
	 */
	public String getError ( ) throws java.rmi.RemoteException
	{
		System.out.println ( "::OrdersEJB:: (getError) " + s_Error );
		return s_Error;
	}


	/**
	 *	Returns page to go to
	 *
	 *	@throws java.rmi.RemoteException
	 *	@returns Page to go to after processing
	 */
	public String getGoto ( ) throws java.rmi.RemoteException
	{
		System.out.println ( "::OrdersEJB:: (getGoto) " + s_Goto );
		return s_Goto;
	}


	/**
	 *	Returns array of String attributes that need to be set for the
	 *	session.
	 *
	 *	@throws java.rmi.RemoteException
	 *	@returns Array of strings of session attributes to set
	 */
	public String[] getSessionAttributes ( ) throws java.rmi.RemoteException
	{
		System.out.println ( "::OrdersEJB:: (getSessionAttributes)" );
		return null;
	}


	/**
	 *	Composes string for SQL order query statement
	 *
	 *	@returns SQL insert statement to query order table
	 */
	private String getOrderQuery ( )
	{
		System.out.println ( "::OrdersEJB:: (getOrderQuery)" );
        return "SELECT * FROM " +
        		"FT_ORDER WHERE " +
        		"Customer = '" + s_Username + "' " +
        		"ORDER BY Date, Time";
	}


	/**
	 *	Composes string for SQL showing query statement
	 *
	 *	@returns SQL insert statement to query showing table
	 */
	private String getShowingQuery ( )
	{
		System.out.println ( "::OrdersEJB:: (getShowingQuery) " + s_ShowingID );
        return "SELECT t.Name, r.Name, m.Title, s.Date, s.Time FROM " +
        		"FT_SHOWING s, FT_THEATER t, FT_ROOM r, FT_MOVIE m WHERE " +
        		"s.RoomID = r.ID AND " +
        		"s.MovieID = m.ID AND " +
        		"r.TheaterID = t.ID AND " +
        		"s.ID = " + s_ShowingID;
	}


	/**
	 *	Processes stored Orders data
	 *
	 *	@throws java.rmi.RemoteException
	 *	@returns Success or Failure of the operation
	 */
    public String process ( ) throws java.rmi.RemoteException
    {
		System.out.println ( "::OrdersEJB:: (process) Begin." );

        if ( s_Username == null )
        {
        	s_Goto = s_GotoFailure;
        	return null;
		}

        StringBuffer message = new StringBuffer ( );

        Connection conn = null;
        Statement stmt = null;
        Statement sstmt = null;
        ResultSet result = null;
        ResultSet sresult = null;

        System.out.println ( "::OrdersEJB:: (process) Starting Datasource access." );

        try
        {
			conn = DBUtil.getDBConnection ( );
            stmt = conn.createStatement ( );
            result = stmt.executeQuery ( this.getOrderQuery ( ) );
			System.out.println ( "::OrdersEJB:: (process) Order query executed." );
     		ResultSetMetaData rsmd = result.getMetaData();
     		int cols = rsmd.getColumnCount();
			result.last ( );
			int rows = result.getRow ( );
			System.out.println ( "::OrdersEJB:: (process) Got " + rows + " results." );
			String row = new String ( );
			String srow = new String ( );
			int i, j;
			int scols;

			message.append ( this.getFormatBeginning ( ) );

			if ( rows == 0 )
			{
				System.out.println ( "::OrdersEJB:: (process) No orders, returning." );
				message.append ( this.getNoOrderRow ( ) );
			}
			else
			{
				message.append ( this.getFormatFields ( ) );
				result.first ( );
				System.out.println ( "::OrdersEJB:: (process) Starting order rows." );
				do
				{
					System.out.println ( "::OrdersEJB:: (process) Start with order row." );
					row = "";
					for ( i = 1; i <= cols; i++ )
						row = row + result.getString ( i ) + this.getDelimiter ( );

					this.storeOrderRow ( row );

            		sstmt = conn.createStatement ( );
					sresult = sstmt.executeQuery ( this.getShowingQuery ( ) );
					srow = "";
					System.out.println ( "::OrdersEJB:: (process) Showing query executed." );
     				sresult.next ( );
     				ResultSetMetaData srsmd = sresult.getMetaData();
					scols = srsmd.getColumnCount();

					for ( j = 1; j <= scols; j++ )
						srow = srow + sresult.getString ( j ) + this.getDelimiter ( );

					message.append ( this.getFormattedRow ( srow ) );
					System.out.println ( "::OrdersEJB:: (process) Done with order row." );
				} while ( result.next ( ) );
				System.out.println ( "::OrdersEJB:: (process) Done with order rows." );
			}

			message.append ( this.getFormatEnding ( ) );
        }
        catch ( Exception e )
        {
            //e.printStackTrace ( );
			System.out.println ( "::OrdersEJB:: (process) Catch failure - No orders, returning." );
			message.append ( this.getNoOrderRow ( ) );
        }
		finally
		{
			DBUtil.closeConnection ( conn );
			DBUtil.closeStatement ( stmt );
			DBUtil.closeStatement ( sstmt );
			DBUtil.closeResultSet ( result );
			DBUtil.closeResultSet ( sresult );
		}

		System.out.println ( "::OrdersEJB:: (process) Returning: " + message.toString ( ) );
        s_Goto = s_GotoSuccess;
        return message.toString ( );
    }


	/**
	 *	Returns formatted date string from yyyy/mm/dd formatted string.
	 *
	 *	@param date in yyyy/mm/dd format
	 *	@returns Date in Month Date, Year format
	 */
	private String getFormattedDate ( String date )
	{
		System.out.println ( "::OrdersEJB:: (getFormattedDate)" );
		String year = s_Date.substring ( 2, 4 );
		String month = s_Date.substring ( 5, 7 );
		String day = s_Date.substring ( 8, 10 );

		return month + "-" + day + "-" + year;
	}


	/**
	 *	Returns formatted row for the case where there are no orders.
	 *
	 *	@returns HTML code for non-order row
	 */
	private String getNoOrderRow ( )
	{
		System.out.println ( "::OrdersEJB:: (getNoOrderRow)" );
		return	"<tr>" +
				"<td class = Error align = center colspan = 20>" +
				"No orders have been made by this user." +
				"</td>" +
				"</tr>";
	}


	/**
	 *	Returns formatted row for the case where there are no orders.
	 *
	 *	@param row Row string of result from showing query
	 *	@returns HTML code for non-order row
	 */
	private String getFormattedRow ( String row )
	{
		System.out.println ( "::OrdersEJB:: (getFormattedRow) " + row );
		String[] rowsplit = row.split ( this.getDelimiter ( ) );
		int i = 0;

		s_TheaterName 	= rowsplit[i];
		s_RoomName 		= rowsplit[++i];
		s_MovieTitle 	= rowsplit[++i];
		s_ShowingDate 	= rowsplit[++i];
		s_ShowingTime 	= rowsplit[++i];

		return	"<tr>" +
				"<td class = MovieTime align = left>" +
				this.getFormattedDate ( s_Date ) +
				"</td>" +
				"<td class = MovieTime align = left>" +
				this.getFormattedTime ( s_Time ) +
				"</td>" +
				"<td class = MovieTime align = left>" +
				s_MovieTitle +
				"</td>" +
				"<td class = MovieTime align = left>" +
				s_TheaterName + ", in " + s_RoomName +
				"</td>" +
				"<td class = MovieTime align = center>" +
				s_Tickets +
				"</td>" +
				"<td class = MovieTime align = left>" +
				this.getFormattedDate ( s_ShowingDate ) +
				"</td>" +
				"<td class = MovieTime align = left>" +
				this.getFormattedTime ( s_ShowingTime ) +
				"</td>" +
				( s_Cancelled.equals ( "0" ) ?
				"<td class = MovieTime align = left>" +
				"Complete <a class = Error href = \"movie?cancelorder=" + s_OrderID + "&showingorder=" + s_ShowingID + "&ticketsorder=" + s_Tickets + "\">(Cancel)</a>" +
				"</td>" +
				"</tr>" :
				"<td class = Error align = left>" +
				"Cancelled" +
				"</td>" +
				"</tr>");
	}


	/**
	 *	Returns formatted movie table string
	 *
	 *	@returns HTML code for theater table
	 */
	private String getFormattedTheater ( )
	{
		System.out.println ( "::OrdersEJB:: (getFormattedTheater)" );

		return 	"<tr>" +
				"<td class = MovieTime align = center>" +
				"Playing in " + s_RoomName + " at " + s_TheaterName +
				"</td>" +
				"</tr>" +
				"<tr>" +
				"</tr>" +
				"</table><br>";
	}


	/**
	 *	Returns formatted time string in hh:mm:ss format.
	 *
	 *	@param time in hh:mm:ss format
	 *	@returns Time in 12-hour HH:MM am/pm format
	 */
	private String getFormattedTime ( String time )
	{
		System.out.println ( "::OrdersEJB:: (getFormattedTime)" );
		int hour = ( new Integer ( time.substring ( 0, 2 ) ).intValue ( ) );
		if ( hour >= 12 )
		{
			if ( hour > 12 )
				hour -= 12;
			return "" + ( hour < 10 ? "0" : "" ) + hour + time.substring ( 2, 5 ) + "pm";
		}
		else
		{
			if ( hour == 0 )
				hour = 12;
			return "" + ( hour < 10 ? "0" : "" ) + hour + time.substring ( 2, 5 ) + "am";
		}
	}


	/**
	 *	Returns the beginning of the order history display.
	 *
	 *	@returns HTML code for beginning display
	 */
	private String getFormatBeginning ( )
	{
		System.out.println ( "::OrdersEJB:: (getFormatBeginning)" );
		return	"<br><table align = center class = MenuTable width = 100% cellpadding = 3>" +
				"<tr>" +
				"<td class = TheaterName align = center colspan = 20>" +
				"Order History for <b>" + s_Username + "</b>" +
				"</td>" +
				"</tr>";
	}


	/**
	 *	Returns the fields.
	 *
	 *	@returns HTML code for beginning display
	 */
	private String getFormatFields ( )
	{
		System.out.println ( "::OrdersEJB:: (getFormatFields)" );
		return	"<tr>" +
				"<td class = MovieTime align = left><b>" +
				"Date" +
				"</b></td>" +
				"<td class = MovieTime align = left><b>" +
				"Time" +
				"</b></td>" +
				"<td class = MovieTime align = left><b>" +
				"Movie Title" +
				"</b></td>" +
				"<td class = MovieTime align = left><b>" +
				"Theater" +
				"</b></td>" +
				"<td class = MovieTime align = left><b>" +
				"Tickets" +
				"</b></td>" +
				"<td class = MovieTime align = left><b>" +
				"Showing Date" +
				"</b></td>" +
				"<td class = MovieTime align = left><b>" +
				"Showing Time" +
				"</b></td>" +
				"<td class = MovieTime align = left><b>" +
				"Status" +
				"</b></td>" +
				"</tr>";
	}


	/**
	 *	Returns the ending of the order history display.
	 *
	 *	@returns HTML code for ending display
	 */
	private String getFormatEnding ( )
	{
		System.out.println ( "::OrdersEJB:: (getFormatEnding)" );
		return	"</table>";
	}


	/**
	 *	Takes order query result string and stores the variables
	 *
	 *	@param row Row string of result from order query
	 */
	private void storeOrderRow ( String row )
	{
		System.out.println ( "::OrdersEJB:: (storeOrderRow) " + row );
		String[] rowsplit = row.split ( this.getDelimiter ( ) );
		int i = 0;

		s_OrderID 	= rowsplit[i];
		s_Username 	= rowsplit[++i];
		s_Date 		= rowsplit[++i];
		s_Time 		= rowsplit[++i];
		s_ShowingID = rowsplit[++i];
		s_Tickets 	= rowsplit[++i];
		s_Cancelled	= rowsplit[++i];

		return;
	}


    /**
     * Creates a bean. Required by EJB spec.
     * @exception throws CreateException.
     */
    public void ejbCreate() throws java.rmi.RemoteException, javax.ejb.CreateException {
        System.out.println("ejbCreate() on obj " + this);
    }

    /**
     * Removes the bean. Required by EJB spec.
     */
    public void ejbRemove() {
        System.out.println("ejbRemove() on obj " + this);
    }

    /**
     * Loads the state of the bean from secondary storage. Required by EJB spec.
     */
    public void ejbActivate() {
        System.out.println("ejbActivate() on obj " + this);
    }

    /**
     * Serializes the state of the bean to secondary storage. Required by EJB spec.
     */
    public void ejbPassivate() {
        System.out.println("ejbPassivate() on obj " + this);
    }

    /**
     * Sets the session context. Required by EJB spec.
     * @param ctx A SessionContext object.
     */
    public void setSessionContext(javax.ejb.SessionContext ctx) {
        m_ctx = ctx;
    }
}
