/**
 * Copyright  2002 Sun Microsystems, Inc. All rights reserved.
 */

package edu.cmu.fantango.ejb;

import java.util.*;
import java.io.*;
import javax.naming.*;
import java.sql.*;
import javax.sql.*;
import javax.servlet.*;
import javax.servlet.http.*;

/**
 *	Static class methods for validating initial form contents
 */
public class RegisterValidator
{
	/**
	 *	Validate form contents
	 *
	 *	@exception Throws ServletException and IOException
	 *	@returns Validity of contents
	 */
    public static boolean Validate ( HttpServletRequest request, HttpServletResponse response )
        throws ServletException, IOException
	{
		System.out.println ( "::RegisterValidator:: (Validate)" );
        // Get localstrings.properties strings
		String errorMsg = new String ( );
        try
        {
			ResourceBundle rb = ResourceBundle.getBundle ( "LocalStrings", Locale.getDefault ( ) );

			// Check for omissions and errors
			if ( request.getParameter ( "username" ).length ( ) == 0 )
				errorMsg = rb.getString ( "invalid_user" );
			else if ( request.getParameter ( "password" ).length ( ) == 0 )
				errorMsg = rb.getString ( "invalid_pass" );
			else if ( ! request.getParameter ( "password" ).equals ( request.getParameter ( "passwordrepeat" ) ) )
				errorMsg = rb.getString ( "invalid_passr" );
			else if ( request.getParameter ( "fname" ).length ( ) == 0 )
				errorMsg = rb.getString ( "invalid_fname" );
			else if ( ! ValidateName ( request.getParameter ( "fname" ) ) || ! ValidAlphabetical ( request.getParameter ( "fname" ) ) )
				errorMsg = rb.getString ( "invalid_fname" );
			else if ( request.getParameter ( "lname" ).length ( ) == 0 )
				errorMsg = rb.getString ( "invalid_lname" );
			else if ( ! ValidateName ( request.getParameter ( "lname" ) ) || ! ValidAlphabetical ( request.getParameter ( "lname" ) ) )
				errorMsg = rb.getString ( "invalid_lname" );
			else if ( request.getParameter ( "email" ).length ( ) == 0 )
				errorMsg = rb.getString ( "invalid_email" );
			else if ( ! ValidateEmail ( request.getParameter ( "email" ) ) )
				errorMsg = rb.getString ( "invalid_email" );
			else
			{
				System.out.println ( "::RegisterValidator:: (Validate) Succeeded." );
				errorMsg = "&nbsp;";
				return true;
			}
		}
		catch ( Exception e )
		{
			errorMsg = "We are sorry, but we have encountered an undefined error.";
		}

		System.out.println ( "::RegisterValidator:: (Validate) Failed: " + errorMsg );
		request.setAttribute ( "error", errorMsg );
		request.setAttribute ( "goto", "/Register.jsp" );

		return false;
	}


    /**
     * Checks alphabetical validity of string
     * @param s The string to check
     * @return Validity
     */
	public static boolean ValidAlphabetical ( String s )
	{
		for ( int i = 0; i < s.length ( ); i++ )
			if ( ( int ) s.charAt ( i ) < 65 || ( int ) s.charAt ( i ) > 122 )
				return false;
		return true;
	}

	/**
	 *	Validate email address
	 *
	 *	@returns Validity of email string
	 */
    private static boolean ValidateEmail ( String email )
    {
		System.out.println ( "::RegisterValidator:: (ValidateEmail)" );
		int at = email.indexOf ( '@' );
		int dot = email.indexOf ( '.' );
		System.out.println ( "Email: |" + email + "|" );
		// Bare minimum is something like a@b.c
		return ( ! ( at == -1 || dot == -1 || at > dot || ( dot - at ) <= 1 || ( email.length ( ) - dot ) <= 1 ) );
	}


	/**
	 *	Validate string
	 *
	 *	@returns Validity of name string
	 */
	private static boolean ValidateName ( String name )
	{
		System.out.println ( "::RegisterValidator:: (ValidateName)" );

		for ( int i = 0; i < 10; i++ )
		{
			if ( name.indexOf ( "" + i ) != -1 )
				return false;
		}

		return true;
	}

}
