/**
 * Copyright  2002 Sun Microsystems, Inc. All rights reserved.
 */

package edu.cmu.fantango.ejb;

import java.util.*;
import java.io.*;
import javax.naming.*;
import java.sql.*;
import javax.sql.*;

import edu.cmu.fantango.util.*;

/**
 * A simple stateless bean for the Showing application. This bean implements one
 * business method as declared by the remote interface.
 */
public class ShowingEJB implements javax.ejb.SessionBean
{
    private  javax.ejb.SessionContext m_ctx = null;

    /**
     *	Stored showing id
     */
	private String s_ShowingID;

    /**
     *	Stored theater name
     */
	private String s_TheaterName;

    /**
     *	Stored room name
     */
	private String s_RoomName;

    /**
     *	Stored movie title
     */
	private String s_MovieTitle;

    /**
     *	Stored running time
     */
	private String s_MovieLength;

    /**
     *	Stored room seats
     */
	private String s_RoomSeats;

    /**
     *	Stored room seats available
     */
	private String s_SeatsAvail;

    /**
     *	Stored room seats
     */
	private String s_Date;

    /**
     *	Stored room seats
     */
	private String s_Time;

    /**
     *	Stored tickets
     */
	private String s_Tickets;

    /**
     *	Stored cc num
     */
	private String s_CCNum;

    /**
     *	Stored cc month
     */
	private String s_CCMonth;

    /**
     *	Stored cc year
     */
	private String s_CCYear;

    /**
     *	Stores result of query in String
     */
	private String s_Result;

    /**
     *	Stores delimiter
     */
	private String s_Delimiter;

    /**
     *	Parameters the bean needs
     */
	private String[] s_Parameters;

    /**
     *	Attributes the bean needs
     */
	private String[] s_Attributes;

    /**
     *	Store error
     */
	private String s_Error;

    /**
     *	Page to load on success
     */
	private static final String s_Goto = "/Showing.jsp";

    /**
     *	Returns the delimiter used for form field separation
     *
     *	@returns Delimiter used for form field separation
     */
	private String getDelimiter ( )
	{
		return s_Delimiter;
	}


    /**
     *	Required by EJB spec.
     */
    public void Showing ( ) {
    }


	/**
	 *	Initializes Showing class to store and process
	 *	showing information.
	 *
	 *	@throws java.rmi.RemoteException
	 */
    public void Init ( ) throws java.rmi.RemoteException
    {
		System.out.println ( "::ShowingEJB:: (Init)" );
		s_Parameters = new String[5];
		s_Parameters[0] = "showing";
		s_Parameters[1] = "tickets";
		s_Parameters[2] = "ccnum";
		s_Parameters[3] = "ccmonth";
		s_Parameters[4] = "ccyear";
		s_Attributes = new String[1];
		s_Attributes[0] = "error";
		s_Error = "&nbsp;";
		s_Delimiter = "__";
    }


	/**
	 *	Initializes parameters from forms (via request)
	 *
	 *	@param	params Values to assign to needed parameters
	 *	@throws java.rmi.RemoteException
	 */
    public void InitParams ( String[] params ) throws java.rmi.RemoteException
    {
		System.out.println ( "::ShowingEJB:: (InitParams)" );
		s_ShowingID = params[0];
		s_Tickets	= params[1];
		s_CCNum		= params[2];
		s_CCMonth	= params[3];
		s_CCYear	= params[4];
    }


	/**
	 *	Initializes attributes from request.
	 *
	 *	@param	attribs Values to assign to needed attributes
	 *	@throws java.rmi.RemoteException
	 */
    public void InitAttribs ( String[] attribs ) throws java.rmi.RemoteException
    {
		System.out.println ( "::ShowingEJB:: (InitAttribs)" );
		s_Error = attribs[0];
		if ( s_Error == null )
			s_Error = "&nbsp;";
    }


	/**
	 *	Returns array of String parameters for the servlet to get
	 *	from request form fields.
	 *
	 *	@throws java.rmi.RemoteException
	 *	@returns Array of strings of parameter names
	 */
	public String[] getParameters ( ) throws java.rmi.RemoteException
	{
		System.out.println ( "::ShowingEJB:: (getParameters)" );
		return s_Parameters;
	}


	/**
	 *	Returns array of String attributes for the servlet to get
	 *	from request.
	 *
	 *	@throws java.rmi.RemoteException
	 *	@returns Array of strings of attribute names
	 */
	public String[] getAttributes ( ) throws java.rmi.RemoteException
	{
		System.out.println ( "::ShowingEJB:: (getAttributes)" );
		return s_Attributes;
	}


	/**
	 *	Returns error state
	 *
	 *	@throws java.rmi.RemoteException
	 *	@returns Possible errors encountered
	 */
	public String getError ( ) throws java.rmi.RemoteException
	{
		System.out.println ( "::ShowingEJB:: (getError) " + s_Error );
		return s_Error;
	}


	/**
	 *	Returns page to go to
	 *
	 *	@throws java.rmi.RemoteException
	 *	@returns Page to go to after processing
	 */
	public String getGoto ( ) throws java.rmi.RemoteException
	{
		System.out.println ( "::ShowingEJB:: (getGoto) " + s_Goto );
		return s_Goto;
	}


	/**
	 *	Returns array of String attributes that need to be set for the
	 *	session.
	 *
	 *	@throws java.rmi.RemoteException
	 *	@returns Array of strings of session attributes to set
	 */
	public String[] getSessionAttributes ( ) throws java.rmi.RemoteException
	{
		System.out.println ( "::ShowingEJB:: (getSessionAttributes)" );
		return null;
	}


	/**
	 *	Composes string for SQL showing query statement
	 *
	 *	@returns SQL insert statement to query showing table
	 */
	private String getQuery ( )
	{
		System.out.println ( "::ShowingEJB:: (getQuery)" );
        return "SELECT t.Name, r.Name, m.Title, m.Length, r.Seats, (r.Seats - s.Sold) AS AVAIL, s.Date, s.Time FROM " +
        		"FT_SHOWING s, FT_THEATER t, FT_ROOM r, FT_MOVIE m WHERE " +
        		"s.RoomID = r.ID AND " +
        		"s.MovieID = m.ID AND " +
        		"r.TheaterID = t.ID AND " +
        		"s.ID = " + s_ShowingID;
	}


	/**
	 *	Processes stored Showing data
	 *
	 *	@throws java.rmi.RemoteException
	 *	@returns Success or Failure of the operation
	 */
    public String process ( ) throws java.rmi.RemoteException
    {
		System.out.println ( "::ShowingEJB:: (process) Begin." );
        StringBuffer message = new StringBuffer ( );

        Connection conn = null;
        Statement stmt = null;
        ResultSet result = null;

        System.out.println ( "::ShowingEJB:: (process) Starting Datasource access." );

        try
        {
			conn = DBUtil.getDBConnection ( );
            stmt = conn.createStatement ( );
            result = stmt.executeQuery ( this.getQuery ( ) );
			System.out.println ( "::ShowingEJB:: (process) Query executed." );
     		ResultSetMetaData rsmd = result.getMetaData();
     		int cols = rsmd.getColumnCount();
			result.next ( );
			int i = 0;
			s_TheaterName 	= result.getString ( ++i );
			s_RoomName 		= result.getString ( ++i );
			s_MovieTitle 	= result.getString ( ++i );
			s_MovieLength	= result.getString ( ++i );
			s_RoomSeats 	= result.getString ( ++i );
			s_SeatsAvail 	= result.getString ( ++i );
			s_Date 			= result.getString ( ++i );
			s_Time 			= result.getString ( ++i );

			message.append ( this.getFormattedTable ( ) );
        }
        catch ( Exception e )
        {
            //e.printStackTrace ( );
			System.out.println ( "::ShowingEJB:: (process) Unknown error" );
			return "Undefined error.";
        }
		finally
		{
			DBUtil.closeConnection ( conn );
			DBUtil.closeStatement ( stmt );
			DBUtil.closeResultSet ( result );
		}

		System.out.println ( "::ShowingEJB:: (process) Returning: " + message.toString ( ) );
        return message.toString ( );
    }


	/**
	 *	Returns formatted date string from yyyy/mm/dd formatted string.
	 *
	 *	@returns Date in Month Date, Year format
	 */
	private String getFormattedDate ( )
	{
		System.out.println ( "::ShowingEJB:: (getFormattedDate)" );
		String year = s_Date.substring ( 0, 4 );
		int mnum = ( new Integer ( s_Date.substring ( 5, 7 ) ) ).intValue ( );
		String day = s_Date.substring ( 8, 10 );
		String month = ( mnum == 1 ? "January" :
						 mnum == 2 ? "February" :
						 mnum == 3 ? "March" :
						 mnum == 4 ? "April" :
						 mnum == 5 ? "May" :
						 mnum == 6 ? "June" :
						 mnum == 7 ? "July" :
						 mnum == 8 ? "August" :
						 mnum == 9 ? "September" :
						 mnum == 10 ? "October" :
						 mnum == 11 ? "November" :
						 "December" );

		return month + " " + day + ", " + year;
	}


	/**
	 *	Returns formatted movie table string
	 *
	 *	@returns HTML code for movie table
	 */
	private String getFormattedMovie ( )
	{
		System.out.println ( "::ShowingEJB:: (getFormattedMovie)" );
		String retString = new String ( );
		retString = "";

		retString = retString + "<br><table align = center class = MenuTable width = 600 cellpadding = 3>";
		retString = retString + "<tr>";
		retString = retString + "<td class = TheaterName align = center>";
		retString = retString + s_MovieTitle;
		retString = retString + "</td>";
		retString = retString + "</tr>";
		retString = retString + "<tr>";
		retString = retString + "<td class = MovieTime align = center>";
		retString = retString + this.getFormattedDate ( );
		retString = retString + " ";
		retString = retString + this.getFormattedTime ( );
		retString = retString + "</td>";
		retString = retString + "</tr>";

		return retString;
	}


	/**
	 *	Returns formatted movie table string
	 *
	 *	@returns HTML code for theater table
	 */
	private String getFormattedTheater ( )
	{
		System.out.println ( "::ShowingEJB:: (getFormattedTheater)" );
		int min = ( new Integer ( s_MovieLength ) ).intValue ( );
		int hour = 0;
		while ( min >= 60 )
		{
			++hour;
			min -= 60;
		}

		return 	"<tr>" +
				"<td class = MovieTime align = center>" +
				"Playing in " + s_RoomName + " at " + s_TheaterName +
				"</td>" +
				"</tr>" +
				"<tr>" +
				"<td class = MovieTime align = center>" +
				hour + "h " + min + "m" +
				"</td>" +
				"</tr>" +
				"</table><br>";
	}


	/**
	 *	Returns formatted time string in hh:mm:ss format.
	 *
	 *	@returns Time in 12-hour HH:MM am/pm format
	 */
	private String getFormattedTime ( )
	{
		System.out.println ( "::ShowingEJB:: (getFormattedTime)" );
		int hour = ( new Integer ( s_Time.substring ( 0, 2 ) ).intValue ( ) );
		if ( hour >= 12 )
		{
			if ( hour > 12 )
				hour -= 12;
			return "" + ( hour < 10 ? "0" : "" ) + hour + s_Time.substring ( 2, 5 ) + "pm";
		}
		else
		{
			if ( hour == 0 )
				hour = 12;
			return "" + ( hour < 10 ? "0" : "" ) + hour + s_Time.substring ( 2, 5 ) + "am";
		}
	}


	/**
	 *	Returns form for purchase of a showing.
	 *
	 *	@returns HTML form
	 */
	private String getFormattedForm ( )
	{
		System.out.println ( "::ShowingEJB:: (getFormattedForm)" );
		return	"<form method=\"POST\" action=\"/movie/movie\">" +
				"<table align = center>" +
				"<tr>" +
				"<td align=center class = Error>" +
				s_Error +
				"</td>" +
				"</tr>" +
				"<tr>" +
				"<td width = 475 class = FormText align = center>" +
				"<input type=\"text\" name=\"tickets\" size=\"2\" maxlength=3 value=\"" + ( s_Tickets != null ? s_Tickets : "0" ) + "\">" +
				"</td>" +
				"</tr>" +
				"<tr>" +
				"<td width = 475 class = MovieTime align = center>" +
				"(" + s_SeatsAvail + " Available)" +
				"</td>" +
				"</tr>" +

				"<tr>" +
				"<td width = 475 align=\"center\">" +
				"Credit Card # and Expiration Month/Year" +
				"</td>" +
				"</tr>" +

				"<tr>" +
				"<td width = 475 class = FormText align = center>" +
				"<select name = \"cctype\">" +
				"<option class = FormText value = \"ccvisa\">Visa</option>" +
				"<option class = FormText value = \"ccmastercard\">Mastercard</option>" +
				"</select>" +
				"<input type=\"text\" name=\"ccnum\" size=\"13\" maxlength=19 value=\"" + ( s_CCNum != null ? s_CCNum : "" ) + "\">" +
				"<input type=\"text\" name=\"ccmonth\" size=\"1\" maxlength=2 value=\"" + ( s_CCMonth != null ? s_CCMonth : "" ) + "\">/" +
				"<input type=\"text\" name=\"ccyear\" size=\"1\" maxlength=2 value=\"" + ( s_CCYear != null ? s_CCYear : "" ) + "\">" +
				"</td>" +
				"</tr>" +

				"<tr>" +
				"<td width = 475 align=\"center\">" +
				"<input type=\"submit\" value=\"Purchase\" name=\"Process\" class = FormContents>" +
				"</td>" +
				"</tr>" +
				"</table>" +
				"<input type=\"hidden\" name=\"showing\" value=\"" + s_ShowingID + "\">" +
				"<input type=\"hidden\" name=\"available\" value=\"" + s_SeatsAvail + "\">" +
				"</form>";
	}


	/**
	 *	Returns full page html code for showing.
	 *
	 *	@returns HTML code for page
	 */
	private String getFormattedTable ( )
	{
		System.out.println ( "::ShowingEJB:: (getFormattedTable)" );
		return	this.getFormattedMovie ( ) +
				this.getFormattedTheater ( ) +
				this.getFormattedForm ( );
	}


    /**
     * Creates a bean. Required by EJB spec.
     * @exception throws CreateException.
     */
    public void ejbCreate() throws java.rmi.RemoteException, javax.ejb.CreateException {
        System.out.println("ejbCreate() on obj " + this);
    }

    /**
     * Removes the bean. Required by EJB spec.
     */
    public void ejbRemove() {
        System.out.println("ejbRemove() on obj " + this);
    }

    /**
     * Loads the state of the bean from secondary storage. Required by EJB spec.
     */
    public void ejbActivate() {
        System.out.println("ejbActivate() on obj " + this);
    }

    /**
     * Serializes the state of the bean to secondary storage. Required by EJB spec.
     */
    public void ejbPassivate() {
        System.out.println("ejbPassivate() on obj " + this);
    }

    /**
     * Sets the session context. Required by EJB spec.
     * @param ctx A SessionContext object.
     */
    public void setSessionContext(javax.ejb.SessionContext ctx) {
        m_ctx = ctx;
    }
}
