/**
 * Copyright  2002 Sun Microsystems, Inc. All rights reserved.
 */

package edu.cmu.fantango.ejb;

import java.util.*;
import java.io.*;
import javax.naming.*;
import java.sql.*;
import javax.sql.*;

import edu.cmu.fantango.util.*;

/**
 * A simple stateless bean for the TheaterList application. This bean implements one
 * business method as declared by the remote interface.
 */
public class TheaterListEJB implements javax.ejb.SessionBean
{
    private  javax.ejb.SessionContext m_ctx = null;

    /**
     *	Stores result of query in String
     */
	private String s_Result;

    /**
     *	Stores delimiter
     */
	private String s_Delimiter;

    /**
     *	Session Attributes the bean needs
     */
	private String[] s_SAttributes;

    /**
     *	Store error
     */
	private String s_Error;

    /**
     *	Page to load
     */
	private static final String s_Goto = "/SearchTheaters.jsp";

    /**
     *	Required by EJB spec.
     */
    public void TheaterList ( ) {
    }


	/**
	 *	Initializes TheaterList class to store and process
	 *	showing information.
	 *
	 *	@throws java.rmi.RemoteException
	 */
    public void Init ( ) throws java.rmi.RemoteException
    {
		System.out.println ( "::TheaterListEJB:: (Init)" );
		s_Delimiter = "__";
    }


    /**
     *	Returns the delimiter used for form field separation
     *
     *	@returns Delimiter used for form field separation
     */
	private String getDelimiter ( )
	{
		return s_Delimiter;
	}


	/**
	 *	Returns error state
	 *
	 *	@throws java.rmi.RemoteException
	 *	@returns Possible errors encountered
	 */
	public String getError ( ) throws java.rmi.RemoteException
	{
		System.out.println ( "::TheaterListEJB:: (getError) " + s_Error );
		return s_Error;
	}


	/**
	 *	Returns page to go to
	 *
	 *	@throws java.rmi.RemoteException
	 *	@returns Page to go to after processing
	 */
	public String getGoto ( ) throws java.rmi.RemoteException
	{
		System.out.println ( "::TheaterListEJB:: (getGoto) " + s_Goto );
		return s_Goto;
	}


	/**
	 *	Composes string for SQL theater query statement
	 *
	 *	@returns SQL insert statement to query theater table
	 */
	private String getTheaterQuery ( )
	{
		System.out.println ( "::TheaterListEJB:: (getTheaterQuery)" );
        return "SELECT * FROM FT_THEATER";
	}


	/**
	 *	Processes stored TheaterList data
	 *
	 *	@throws java.rmi.RemoteException
	 *	@returns Success or Failure of the operation
	 */
    public String process ( ) throws java.rmi.RemoteException
    {
		System.out.println ( "::TheaterListEJB:: (process) Begin." );

        Connection conn = null;
        Statement stmt = null;
        ResultSet result = null;
		String retString = new String ( );

        System.out.println ( "::TheaterListEJB:: (process) Starting Datasource access." );
        try
        {
			conn = DBUtil.getDBConnection ( );
            stmt = conn.createStatement ( );
            result = stmt.executeQuery ( this.getTheaterQuery ( ) );
			System.out.println ( "::TheaterListEJB:: (process) Theater query executed." );

			result.last ( );
			int rows = result.getRow ( );
			result.first ( );

			if ( rows == 0 )
			{
        		System.out.println ( "::TheaterListEJB:: (process) Returning null." );
				retString = null;
			}
			else
			{
				retString = "";

				for ( int i = 0; i < rows; i++ )
				{
					retString = retString + getFormattedList ( result.getString ( 1 ), result.getString ( 2 ) );
					result.next ( );
				}
        		System.out.println ( "::TheaterListEJB:: (process) Returning: " + retString );
			}
        }
        catch ( Exception e )
        {
            //e.printStackTrace ( );
			System.out.println ( "::TheaterListEJB:: (process) Caught exception, returning null." );
			retString = null;
        }
		finally
		{
			DBUtil.closeConnection ( conn );
			DBUtil.closeStatement ( stmt );
			DBUtil.closeResultSet ( result );
		}

		return retString;
    }


	/**
	 *	Returns formatted row for theater option
	 *
	 *	@param id ID String of movie
	 *	@param name Name String of movie
	 *	@returns HTML code for non-order row
	 */
	private String getFormattedList ( String id, String name )
	{
       	System.out.println ( "::TheaterListEJB:: (getFormattedList)" );

		return	"<option class = FormText value = \"" +
				id +
				"\" />" +
				name +
				"</option>";
	}


    /**
     * Creates a bean. Required by EJB spec.
     * @exception throws CreateException.
     */
    public void ejbCreate() throws java.rmi.RemoteException, javax.ejb.CreateException {
        System.out.println("ejbCreate() on obj " + this);
    }

    /**
     * Removes the bean. Required by EJB spec.
     */
    public void ejbRemove() {
        System.out.println("ejbRemove() on obj " + this);
    }

    /**
     * Loads the state of the bean from secondary storage. Required by EJB spec.
     */
    public void ejbActivate() {
        System.out.println("ejbActivate() on obj " + this);
    }

    /**
     * Serializes the state of the bean to secondary storage. Required by EJB spec.
     */
    public void ejbPassivate() {
        System.out.println("ejbPassivate() on obj " + this);
    }

    /**
     * Sets the session context. Required by EJB spec.
     * @param ctx A SessionContext object.
     */
    public void setSessionContext(javax.ejb.SessionContext ctx) {
        m_ctx = ctx;
    }
}
