/*
 * Command.java
 * 
 * Created for HW2 of Analysis of Software Artifacts
 * $Id: Command.java,v 1.2 2004/02/27 02:42:10 ahmedb Exp $
 * $Log: Command.java,v $
 * Revision 1.2  2004/02/27 02:42:10  ahmedb
 * added time variable to indicate when the command was created
 *
 * Revision 1.1  2004/02/26 20:31:42  ahmedb
 * new classes for Replication Management
 *
 * Revision 1.5  2004/02/16 22:04:17  hisashiy
 * Modified/added debug printing. Removed pid handling from Java code. List-mode now handles very first time access to JNDI (it now creates root node).
 *
 */
package edu.cmu.fantango.servlet;

import java.io.Serializable;
import java.net.InetAddress;
import java.net.UnknownHostException;

/**
 * Command class is a representation of a command that is issued
 * by -p option of repman. This class is serializable so that it
 * could be held persistent in other forms (such as fil or JNDI
 * object.)
 * 
 * @author hisashiy
 *
 */
public class Command implements Serializable {

    /**
     * Label of a command. Will be used as a key for finding a command.
     */
    protected String label;

    /**
     * String representation of a command this object represents.
     */
    protected String cmd;
    
    /**
     * Machine or IP address of a command to run (or running) remotely.
     * It would be local machine by default if no arguement is provided.
     */
    protected String machine;

    /**
     * Time when command was created
     * used to make sure we do not kill JBoss command as they start
     */
    protected long time;

    /** 
     * The absolute pathname of ssh.<br>
     * note: use /usr/local/bin/ssh instead of /usr/bin/ssh to avoid password prompt.
     */
    protected static final String SSH_PATH = "/usr/local/bin/ssh";
    protected static final String DIR = "~/repman/";   

    /**
     * Current working directory for leaving a log file.
     */
    protected static final String CWD = System.getProperty("user.dir", "") + "/";
    
    /**
     *
     */
    public Command(String cmd, String label, String machine, long time) {
        String hostname = machine;

        setCommand(cmd);
        setLabel(label);
	setTime(time);

        if(hostname == null){
            try {
                InetAddress adr = InetAddress.getLocalHost();
                hostname = adr.getHostName();
            } catch (UnknownHostException e) {
                hostname="$HOSTNAME";
            }
        } else {
            hostname = machine;
        }
        setMachine(hostname);
    }

    /**
     * Get label for this command object.
     * @return label of the command.
     */
    public String getLabel() {
        return label;
    }

    /**
     * Set label for this command object.
     * @param label a label to specify the program invoked.
     */
    protected void setLabel(String label) {
        this.label = label;
    }

    /**
     * Get string representation of the command.
     * @return string representation of the command.
     */
    public String getCommand() {
        return cmd;
    }

    /**
     * Set string representation of the command.
     * @param string
     */
    protected void setCommand(String cmd) {
        this.cmd = cmd;
    }

    /**
     * Get a machine name or IP address of where the command is running
     * or where command will run.
     * @return machine name of IP address of where command runs/running.
     */
    public String getMachine() {
        return machine;
    }

    /**
     * Set a machine name or IP address of where the command is running
     * or where command will run.
     * @param machine a machine name or IP address
     */
    protected void setMachine(String machine) {
        this.machine = machine;
    }


    /**
     * Get label for this command object.
     * @return label of the command.
     */
    public long getTime() {
        return time;
    }

    /**
     * Set label for this command object.
     * @param label a label to specify the program invoked.
     */
    protected void setTime(long time) {
        this.time = time;
    }

    /**
     * Creates a command line to be executed. Returns a ssh command line
     * including hostname, command, and log file setting.
     * @return string of command line
     */
    public String createCommandLine() {
    String hostname = "$HOSTNAME";
    
    if(machine == null){
        try {
        InetAddress adr = InetAddress.getLocalHost();
        hostname = adr.getHostName();
        } catch (UnknownHostException e) {
                //hostname will be "$HOSTNAME"
        }   
    } else {    
        hostname = machine;
    }   
    
    String cmdLine = SSH_PATH + " "  + hostname + " "
        + DIR + "execute " + cmd + " " + label +" > "
        + DIR + label + ".log" + " 2>&1";
    return cmdLine;
    }

    /**
     * Creates a command line to be executed. Returns a ssh command line
     * including hostname, command, and log file setting.
     * @return string of command line
     */
    public String createKillCommandLine() {
    String hostname = "$HOSTNAME";
    
    if(machine == null){
        try {
        InetAddress adr = InetAddress.getLocalHost();
        hostname = adr.getHostName();
        } catch (UnknownHostException e) {
                //hostname will be "$HOSTNAME"
        }   
    } else {    
        hostname = machine;
    }   
    
    String cmdLine = SSH_PATH + " "  + hostname + " "
        + "kill -9 `cat ~/" + label + "` ; rm ~/" + label;

    return cmdLine;
    }

}   
